from pg_extended import Types
from pg_extended import Core
from pg_extended import UI
from pg_extended import Game
from pg_extended import Util
from pg_extended import Window

from pg_extended.Types import *
from pg_extended.Core import *
from pg_extended.UI import *
from pg_extended.Game import *
from pg_extended.Util import *
from pg_extended.Window import *

import pygame
pygame.init()
